from typing import Optional, Sequence

from promptl_ai import MessageLike
from promptl_ai.bindings.types import _Message

from latitude_sdk.client import Client, CreateLogRequestBody, CreateLogRequestParams, RequestHandler
from latitude_sdk.sdk.errors import ApiError, ApiErrorCodes
from latitude_sdk.sdk.types import Log, SdkOptions
from latitude_sdk.util import Model


class LogOptions(Model):
    project_id: Optional[int] = None
    version_uuid: Optional[str] = None


class CreateLogOptions(LogOptions, Model):
    response: Optional[str] = None


class CreateLogResult(Log, Model):
    pass


class Logs:
    _options: SdkOptions
    _client: Client

    def __init__(self, client: Client, options: SdkOptions):
        self._options = options
        self._client = client

    def _ensure_log_options(self, options: LogOptions):
        if not options.project_id:
            raise ApiError(
                status=404,
                code=ApiErrorCodes.NotFoundError,
                message="Project ID is required",
                response="Project ID is required",
            )

    async def create(
        self, path: str, messages: Sequence[MessageLike], options: Optional[CreateLogOptions] = None
    ) -> CreateLogResult:
        options = CreateLogOptions(**{**dict(self._options), **dict(options or {})})
        self._ensure_log_options(options)
        assert options.project_id is not None

        messages = [_Message.validate_python(message) for message in messages]

        async with self._client.request(
            handler=RequestHandler.CreateLog,
            params=CreateLogRequestParams(
                project_id=options.project_id,
                version_uuid=options.version_uuid,
            ),
            body=CreateLogRequestBody(
                path=path,
                messages=messages,
                response=options.response,
            ),
        ) as response:
            return CreateLogResult.model_validate_json(response.content)



from typing import Optional

from promptl_ai import Promptl, PromptlOptions

from latitude_sdk.client import Client, ClientOptions, RouterOptions
from latitude_sdk.env import env
from latitude_sdk.sdk.evaluations import Evaluations
from latitude_sdk.sdk.logs import Logs
from latitude_sdk.sdk.prompts import Prompts
from latitude_sdk.sdk.types import GatewayOptions, LogSources, SdkOptions
from latitude_sdk.util import Model


class InternalOptions(Model):
    gateway: Optional[GatewayOptions] = None
    source: Optional[LogSources] = None
    retries: Optional[int] = None
    delay: Optional[float] = None
    timeout: Optional[float] = None


class LatitudeOptions(SdkOptions, Model):
    promptl: Optional[PromptlOptions] = None
    internal: Optional[InternalOptions] = None


DEFAULT_INTERNAL_OPTIONS = InternalOptions(
    gateway=GatewayOptions(
        host=env.GATEWAY_HOSTNAME,
        port=env.GATEWAY_PORT,
        ssl=env.GATEWAY_SSL,
        api_version="v3",
    ),
    source=LogSources.Api,
    retries=3,
    delay=0.5,
    timeout=30,
)


DEFAULT_LATITUDE_OPTIONS = LatitudeOptions(
    internal=DEFAULT_INTERNAL_OPTIONS,
)


class Latitude:
    _options: LatitudeOptions
    _client: Client

    promptl: Promptl

    prompts: Prompts
    logs: Logs
    evaluations: Evaluations

    def __init__(self, api_key: str, options: Optional[LatitudeOptions] = None):
        options = LatitudeOptions(**{**dict(DEFAULT_LATITUDE_OPTIONS), **dict(options or {})})
        options.internal = InternalOptions(**{**dict(DEFAULT_INTERNAL_OPTIONS), **dict(options.internal or {})})
        self._options = options

        assert self._options.internal is not None
        assert self._options.internal.gateway is not None
        assert self._options.internal.source is not None
        assert self._options.internal.retries is not None
        assert self._options.internal.delay is not None
        assert self._options.internal.timeout is not None

        self._client = Client(
            ClientOptions(
                api_key=api_key,
                retries=self._options.internal.retries,
                delay=self._options.internal.delay,
                timeout=self._options.internal.timeout,
                source=self._options.internal.source,
                router=RouterOptions(gateway=self._options.internal.gateway),
            )
        )

        self.promptl = Promptl(self._options.promptl)
        self.prompts = Prompts(self._client, self.promptl, self._options)
        self.logs = Logs(self._client, self._options)
        self.evaluations = Evaluations(self._client, self._options)



from typing import List, Optional, Union

from latitude_sdk.client import (
    Client,
    CreateEvaluationResultRequestBody,
    CreateEvaluationResultRequestParams,
    RequestHandler,
    TriggerEvaluationRequestBody,
    TriggerEvaluationRequestParams,
)
from latitude_sdk.sdk.types import EvaluationResult, SdkOptions
from latitude_sdk.util import Model


class TriggerEvaluationOptions(Model):
    evaluation_uuids: Optional[List[str]] = None


class TriggerEvaluationResult(Model):
    evaluations: List[str]


class CreateEvaluationResultOptions(Model):
    result: Union[str, bool, int]
    reason: str


class CreateEvaluationResultResult(EvaluationResult, Model):
    pass


class Evaluations:
    _options: SdkOptions
    _client: Client

    def __init__(self, client: Client, options: SdkOptions):
        self._options = options
        self._client = client

    async def trigger(self, uuid: str, options: Optional[TriggerEvaluationOptions] = None) -> TriggerEvaluationResult:
        options = TriggerEvaluationOptions(**{**dict(self._options), **dict(options or {})})

        async with self._client.request(
            handler=RequestHandler.TriggerEvaluation,
            params=TriggerEvaluationRequestParams(
                conversation_uuid=uuid,
            ),
            body=TriggerEvaluationRequestBody(
                evaluation_uuids=options.evaluation_uuids,
            ),
        ) as response:
            return TriggerEvaluationResult.model_validate_json(response.content)

    async def create_result(
        self, uuid: str, evaluation_uuid: str, options: CreateEvaluationResultOptions
    ) -> CreateEvaluationResultResult:
        options = CreateEvaluationResultOptions(**{**dict(self._options), **dict(options)})

        async with self._client.request(
            handler=RequestHandler.CreateEvaluationResult,
            params=CreateEvaluationResultRequestParams(
                conversation_uuid=uuid,
                evaluation_uuid=evaluation_uuid,
            ),
            body=CreateEvaluationResultRequestBody(
                result=options.result,
                reason=options.reason,
            ),
        ) as response:
            return CreateEvaluationResultResult.model_validate_json(response.content)



import asyncio
from typing import Any, AsyncGenerator, List, Optional, Sequence, Union

from promptl_ai import Adapter, Message, MessageLike, Promptl, ToolMessage, ToolResultContent
from promptl_ai.bindings.types import _Message

from latitude_sdk.client import (
    ChatPromptRequestBody,
    ChatPromptRequestParams,
    Client,
    ClientEvent,
    GetAllPromptRequestParams,
    GetOrCreatePromptRequestBody,
    GetOrCreatePromptRequestParams,
    GetPromptRequestParams,
    RequestHandler,
    RunPromptRequestBody,
    RunPromptRequestParams,
)
from latitude_sdk.sdk.errors import ApiError, ApiErrorCodes
from latitude_sdk.sdk.types import (
    ChainEvents,
    FinishedResult,
    OnStep,
    OnToolCall,
    OnToolCallDetails,
    Prompt,
    Providers,
    SdkOptions,
    StreamCallbacks,
    StreamEvents,
    StreamTypes,
    ToolCall,
    ToolResult,
    _LatitudeEvent,
)
from latitude_sdk.util import Adapter as AdapterUtil
from latitude_sdk.util import Model

_PROVIDER_TO_ADAPTER = {
    Providers.OpenAI: Adapter.OpenAI,
    Providers.Anthropic: Adapter.Anthropic,
}

_PROMPT_ATTR_TO_ADAPTER_ATTR = {
    "maxTokens": ("max_tokens", [Adapter.OpenAI, Adapter.Anthropic]),
    "topP": ("top_p", [Adapter.OpenAI, Adapter.Anthropic]),
    "topK": ("top_k", [Adapter.OpenAI, Adapter.Anthropic]),
    "presencePenalty": ("presence_penalty", [Adapter.OpenAI, Adapter.Anthropic]),
    "stopSequences": ("stop_sequences", [Adapter.OpenAI, Adapter.Anthropic]),
    "toolChoice": ("tool_choice", [Adapter.OpenAI, Adapter.Anthropic]),
}


class OnToolCallPaused(Exception):
    pass


class PromptOptions(Model):
    project_id: Optional[int] = None
    version_uuid: Optional[str] = None


class GetPromptOptions(PromptOptions, Model):
    pass


class GetPromptResult(Prompt, Model):
    pass


_GetAllPromptResults = AdapterUtil[List[GetPromptResult]](List[GetPromptResult])


class GetAllPromptOptions(PromptOptions, Model):
    pass


class GetOrCreatePromptOptions(PromptOptions, Model):
    prompt: Optional[str] = None


class GetOrCreatePromptResult(Prompt, Model):
    pass


class RunPromptOptions(StreamCallbacks, PromptOptions, Model):
    custom_identifier: Optional[str] = None
    parameters: Optional[dict[str, Any]] = None
    tools: Optional[dict[str, OnToolCall]] = None
    stream: Optional[bool] = None


class RunPromptResult(FinishedResult, Model):
    pass


class ChatPromptOptions(StreamCallbacks, Model):
    tools: Optional[dict[str, OnToolCall]] = None
    stream: Optional[bool] = None


class ChatPromptResult(FinishedResult, Model):
    pass


class RenderPromptOptions(Model):
    parameters: Optional[dict[str, Any]] = None
    adapter: Optional[Adapter] = None


class RenderPromptResult(Model):
    messages: list[MessageLike]
    config: dict[str, Any]


class RenderChainOptions(Model):
    parameters: Optional[dict[str, Any]] = None
    adapter: Optional[Adapter] = None


class RenderChainResult(RenderPromptResult, Model):
    pass


class Prompts:
    _options: SdkOptions
    _client: Client
    _promptl: Promptl

    def __init__(self, client: Client, promptl: Promptl, options: SdkOptions):
        self._options = options
        self._client = client
        self._promptl = promptl

    def _ensure_prompt_options(self, options: PromptOptions):
        if not options.project_id:
            raise ApiError(
                status=404,
                code=ApiErrorCodes.NotFoundError,
                message="Project ID is required",
                response="Project ID is required",
            )

    async def _handle_stream(
        self, stream: AsyncGenerator[ClientEvent, Any], on_event: Optional[StreamCallbacks.OnEvent]
    ) -> FinishedResult:
        uuid = None
        conversation: list[Message] = []
        response = None
        tool_requests: list[ToolCall] = []

        async for stream_event in stream:
            event = None

            if stream_event.event == str(StreamEvents.Latitude):
                event = _LatitudeEvent.validate_json(stream_event.data)
                conversation = event.messages
                uuid = event.uuid

                if event.type == ChainEvents.ProviderCompleted:
                    response = event.response

                elif event.type == ChainEvents.ToolsRequested:
                    tool_requests = event.tools

                elif event.type == ChainEvents.ChainError:
                    raise ApiError(
                        status=400,
                        code=ApiErrorCodes.AIRunError,
                        message=event.error.message,
                        response=stream_event.data,
                    )

            elif stream_event.event == str(StreamEvents.Provider):
                event = stream_event.json()
                event["event"] = StreamEvents.Provider

            else:
                raise ApiError(
                    status=500,
                    code=ApiErrorCodes.InternalServerError,
                    message=f"Unknown stream event: {stream_event.event}",
                    response=stream_event.data,
                )

            if on_event:
                on_event(event)

        if not uuid or not response:
            raise ApiError(
                status=500,
                code=ApiErrorCodes.InternalServerError,
                message="Stream ended without a chain-complete event. Missing uuid or response.",
                response="Stream ended without a chain-complete event. Missing uuid or response.",
            )

        return FinishedResult(uuid=uuid, conversation=conversation, response=response, tool_requests=tool_requests)

    @staticmethod
    def _pause_tool_execution() -> Any:
        raise OnToolCallPaused()

    @staticmethod
    async def _wrap_tool_handler(
        handler: OnToolCall, arguments: dict[str, Any], details: OnToolCallDetails
    ) -> ToolResult:
        tool_result: dict[str, Any] = {"id": details.id, "name": details.name}

        try:
            result = await handler(arguments, details)

            return ToolResult(**tool_result, result=result)
        except Exception as exception:
            if isinstance(exception, OnToolCallPaused):
                raise exception

            return ToolResult(**tool_result, result=str(exception), is_error=True)

    async def _handle_tool_calls(
        self, result: FinishedResult, options: Union[RunPromptOptions, ChatPromptOptions]
    ) -> Optional[FinishedResult]:
        # Seems Python cannot infer the type
        assert result.response.type == StreamTypes.Text and result.tool_requests is not None

        if not options.tools:
            raise ApiError(
                status=400,
                code=ApiErrorCodes.AIRunError,
                message="Tools not supplied",
                response="Tools not supplied",
            )

        for tool_call in result.tool_requests:
            if tool_call.name not in options.tools:
                raise ApiError(
                    status=400,
                    code=ApiErrorCodes.AIRunError,
                    message=f"Tool {tool_call.name} not supplied",
                    response=f"Tool {tool_call.name} not supplied",
                )

        tool_results = await asyncio.gather(
            *[
                self._wrap_tool_handler(
                    options.tools[tool_call.name],
                    tool_call.arguments,
                    OnToolCallDetails(
                        id=tool_call.id,
                        name=tool_call.name,
                        conversation_uuid=result.uuid,
                        messages=result.conversation,
                        pause_execution=self._pause_tool_execution,
                        requested_tool_calls=result.tool_requests,
                    ),
                )
                for tool_call in result.tool_requests
            ],
            return_exceptions=False,
        )

        tool_messages = [
            ToolMessage(
                content=[
                    ToolResultContent(
                        id=tool_result.id,
                        name=tool_result.name,
                        result=tool_result.result,
                        is_error=tool_result.is_error,
                    )
                ]
            )
            for tool_result in tool_results
        ]

        next_result = await self.chat(result.uuid, tool_messages, ChatPromptOptions(**dict(options)))

        return FinishedResult(**dict(next_result)) if next_result else None

    async def get(self, path: str, options: Optional[GetPromptOptions] = None) -> GetPromptResult:
        options = GetPromptOptions(**{**dict(self._options), **dict(options or {})})
        self._ensure_prompt_options(options)
        assert options.project_id is not None

        async with self._client.request(
            handler=RequestHandler.GetPrompt,
            params=GetPromptRequestParams(
                project_id=options.project_id,
                version_uuid=options.version_uuid,
                path=path,
            ),
        ) as response:
            return GetPromptResult.model_validate_json(response.content)

    async def get_all(self, options: Optional[GetAllPromptOptions] = None) -> List[GetPromptResult]:
        options = GetAllPromptOptions(**{**dict(self._options), **dict(options or {})})
        self._ensure_prompt_options(options)
        assert options.project_id is not None

        async with self._client.request(
            handler=RequestHandler.GetAllPrompts,
            params=GetAllPromptRequestParams(
                project_id=options.project_id,
                version_uuid=options.version_uuid,
            ),
        ) as response:
            return _GetAllPromptResults.validate_json(response.content)

    async def get_or_create(
        self, path: str, options: Optional[GetOrCreatePromptOptions] = None
    ) -> GetOrCreatePromptResult:
        options = GetOrCreatePromptOptions(**{**dict(self._options), **dict(options or {})})
        self._ensure_prompt_options(options)
        assert options.project_id is not None

        async with self._client.request(
            handler=RequestHandler.GetOrCreatePrompt,
            params=GetOrCreatePromptRequestParams(
                project_id=options.project_id,
                version_uuid=options.version_uuid,
            ),
            body=GetOrCreatePromptRequestBody(
                path=path,
                prompt=options.prompt,
            ),
        ) as response:
            return GetOrCreatePromptResult.model_validate_json(response.content)

    async def run(self, path: str, options: Optional[RunPromptOptions] = None) -> Optional[RunPromptResult]:
        options = RunPromptOptions(**{**dict(self._options), **dict(options or {})})
        self._ensure_prompt_options(options)
        assert options.project_id is not None

        try:
            async with self._client.request(
                handler=RequestHandler.RunPrompt,
                params=RunPromptRequestParams(
                    project_id=options.project_id,
                    version_uuid=options.version_uuid,
                ),
                body=RunPromptRequestBody(
                    path=path,
                    parameters=options.parameters,
                    custom_identifier=options.custom_identifier,
                    stream=options.stream,
                ),
            ) as response:
                if options.stream:
                    result = await self._handle_stream(response.sse(), options.on_event)
                else:
                    result = RunPromptResult.model_validate_json(response.content)

            if options.tools and result.response.type == StreamTypes.Text and result.tool_requests:
                try:
                    # NOTE: The last sdk.chat called will already call on_finished
                    final_result = await self._handle_tool_calls(result, options)
                    return RunPromptResult(**dict(final_result)) if final_result else None
                except OnToolCallPaused:
                    pass

            if options.on_finished:
                options.on_finished(FinishedResult(**dict(result)))

            return RunPromptResult(**dict(result))

        except Exception as exception:
            if not isinstance(exception, ApiError):
                exception = ApiError(
                    status=500,
                    code=ApiErrorCodes.InternalServerError,
                    message=str(exception),
                    response=str(exception),
                )

            if not options.on_error:
                raise exception

            options.on_error(exception)

            return None

    async def chat(
        self, uuid: str, messages: Sequence[MessageLike], options: Optional[ChatPromptOptions] = None
    ) -> Optional[ChatPromptResult]:
        options = ChatPromptOptions(**{**dict(self._options), **dict(options or {})})

        messages = [_Message.validate_python(message) for message in messages]

        try:
            async with self._client.request(
                handler=RequestHandler.ChatPrompt,
                params=ChatPromptRequestParams(
                    conversation_uuid=uuid,
                ),
                body=ChatPromptRequestBody(
                    messages=messages,
                    stream=options.stream,
                ),
            ) as response:
                if options.stream:
                    result = await self._handle_stream(response.sse(), options.on_event)
                else:
                    result = ChatPromptResult.model_validate_json(response.content)

            if options.tools and result.response.type == StreamTypes.Text and result.tool_requests:
                try:
                    # NOTE: The last sdk.chat called will already call on_finished
                    final_result = await self._handle_tool_calls(result, options)
                    return ChatPromptResult(**dict(final_result)) if final_result else None
                except OnToolCallPaused:
                    pass

            if options.on_finished:
                options.on_finished(FinishedResult(**dict(result)))

            return ChatPromptResult(**dict(result))

        except Exception as exception:
            if not isinstance(exception, ApiError):
                exception = ApiError(
                    status=500,
                    code=ApiErrorCodes.InternalServerError,
                    message=str(exception),
                    response=str(exception),
                )

            if not options.on_error:
                raise exception

            options.on_error(exception)

            return None

    def _adapt_prompt_config(self, config: dict[str, Any], adapter: Adapter) -> dict[str, Any]:
        adapted_config: dict[str, Any] = {}

        # NOTE: Should we delete attributes not supported by the provider?
        for attr, value in config.items():
            if attr in _PROMPT_ATTR_TO_ADAPTER_ATTR and adapter in _PROMPT_ATTR_TO_ADAPTER_ATTR[attr][1]:
                adapted_config[_PROMPT_ATTR_TO_ADAPTER_ATTR[attr][0]] = value
            else:
                adapted_config[attr] = value

        return adapted_config

    async def render(self, prompt: str, options: Optional[RenderPromptOptions] = None) -> RenderPromptResult:
        options = RenderPromptOptions(**{**dict(self._options), **dict(options or {})})
        adapter = options.adapter or Adapter.OpenAI

        result = self._promptl.prompts.render(
            prompt=prompt,
            parameters=options.parameters,
            adapter=adapter,
        )

        return RenderPromptResult(
            messages=result.messages,
            config=self._adapt_prompt_config(result.config, adapter),
        )

    async def render_chain(
        self, prompt: Prompt, on_step: OnStep, options: Optional[RenderChainOptions] = None
    ) -> RenderChainResult:
        options = RenderChainOptions(**{**dict(self._options), **dict(options or {})})
        adapter = options.adapter or _PROVIDER_TO_ADAPTER.get(prompt.provider or Providers.OpenAI, Adapter.OpenAI)

        chain = self._promptl.chains.create(
            prompt=prompt.content,
            parameters=options.parameters,
            adapter=adapter,
        )

        step = None
        response = None
        while not chain.completed:
            step = chain.step(response)
            if not step.completed:
                response = await on_step(step.messages, self._adapt_prompt_config(step.config, adapter))

        assert step is not None

        return RenderChainResult(
            messages=step.messages,
            config=self._adapt_prompt_config(step.config, adapter),
        )



from typing import Callable, Optional, Tuple

from latitude_sdk.client.payloads import (
    ChatPromptRequestParams,
    CreateEvaluationResultRequestParams,
    CreateLogRequestParams,
    GetAllPromptRequestParams,
    GetOrCreatePromptRequestParams,
    GetPromptRequestParams,
    RequestHandler,
    RequestParams,
    RunPromptRequestParams,
    TriggerEvaluationRequestParams,
)
from latitude_sdk.sdk.types import GatewayOptions
from latitude_sdk.util import Model

HEAD_COMMIT = "live"


class RouterOptions(Model):
    gateway: GatewayOptions


class Router:
    options: RouterOptions

    def __init__(self, options: RouterOptions):
        self.options = options

    def resolve(self, handler: RequestHandler, params: RequestParams) -> Tuple[str, str]:
        if handler == RequestHandler.GetPrompt:
            assert isinstance(params, GetPromptRequestParams)

            return "GET", self.prompts(
                project_id=params.project_id,
                version_uuid=params.version_uuid,
            ).prompt(params.path)

        if handler == RequestHandler.GetAllPrompts:
            assert isinstance(params, GetAllPromptRequestParams)

            return "GET", self.prompts(
                project_id=params.project_id,
                version_uuid=params.version_uuid,
            ).all_prompts

        elif handler == RequestHandler.GetOrCreatePrompt:
            assert isinstance(params, GetOrCreatePromptRequestParams)

            return "POST", self.prompts(
                project_id=params.project_id,
                version_uuid=params.version_uuid,
            ).get_or_create

        elif handler == RequestHandler.RunPrompt:
            assert isinstance(params, RunPromptRequestParams)

            return "POST", self.prompts(
                project_id=params.project_id,
                version_uuid=params.version_uuid,
            ).run

        elif handler == RequestHandler.ChatPrompt:
            assert isinstance(params, ChatPromptRequestParams)

            return "POST", self.conversations().chat(params.conversation_uuid)

        elif handler == RequestHandler.CreateLog:
            assert isinstance(params, CreateLogRequestParams)

            return "POST", self.prompts(
                project_id=params.project_id,
                version_uuid=params.version_uuid,
            ).logs

        elif handler == RequestHandler.TriggerEvaluation:
            assert isinstance(params, TriggerEvaluationRequestParams)

            return "POST", self.conversations().evaluate(params.conversation_uuid)

        elif handler == RequestHandler.CreateEvaluationResult:
            assert isinstance(params, CreateEvaluationResultRequestParams)

            return "POST", self.conversations().evaluation_result(params.conversation_uuid, params.evaluation_uuid)

        raise TypeError(f"Unknown handler: {handler}")

    class Conversations(Model):
        chat: Callable[[str], str]
        evaluate: Callable[[str], str]
        evaluation_result: Callable[[str, str], str]

    def conversations(self) -> Conversations:
        base_url = f"{self.options.gateway.base_url}/conversations"

        return self.Conversations(
            chat=lambda uuid: f"{base_url}/{uuid}/chat",
            evaluate=lambda uuid: f"{base_url}/{uuid}/evaluate",
            evaluation_result=lambda conversation_uuid,
            evaluation_uuid: f"{base_url}/{conversation_uuid}/evaluations/{evaluation_uuid}/evaluation-results",
        )

    class Prompts(Model):
        prompt: Callable[[str], str]
        all_prompts: str
        get_or_create: str
        run: str
        logs: str

    def prompts(self, project_id: int, version_uuid: Optional[str]) -> Prompts:
        base_url = f"{self.commits_url(project_id, version_uuid)}/documents"

        return self.Prompts(
            all_prompts=f"{base_url}",
            prompt=lambda path: f"{base_url}/{path}",
            get_or_create=f"{base_url}/get-or-create",
            run=f"{base_url}/run",
            logs=f"{base_url}/logs",
        )

    def commits_url(self, project_id: int, version_uuid: Optional[str]) -> str:
        version_uuid = version_uuid if version_uuid else HEAD_COMMIT

        return f"{self.projects_url(project_id)}/versions/{version_uuid}"

    def projects_url(self, project_id: int) -> str:
        return f"{self.options.gateway.base_url}/projects/{project_id}"



from typing import List, cast

import httpx

from latitude_sdk import TriggerEvaluationOptions, TriggerEvaluationResult
from tests.utils import TestCase, fixtures


class TestTriggerEvaluation(TestCase):
    async def test_success(self):
        conversation_uuid = "conversation-uuid"
        options = TriggerEvaluationOptions(evaluation_uuids=["evaluation-uuid-1", "evaluation-uuid-2"])
        endpoint = f"/conversations/{conversation_uuid}/evaluate"
        endpoint_mock = self.gateway_mock.post(endpoint).mock(
            return_value=httpx.Response(200, json=fixtures.EVALUATIONS_RESPONSE)
        )

        result = await self.sdk.evaluations.trigger(conversation_uuid, options)
        request, _ = endpoint_mock.calls.last

        self.assert_requested(
            request,
            method="POST",
            endpoint=endpoint,
            body={
                "evaluationUuids": options.evaluation_uuids,
            },
        )
        self.assertEqual(endpoint_mock.call_count, 1)
        self.assertEqual(result, TriggerEvaluationResult(evaluations=fixtures.EVALUATIONS))

    async def test_fails(self):
        conversation_uuid = "conversation-uuid"
        endpoint = f"/conversations/{conversation_uuid}/evaluate"
        endpoint_mock = self.gateway_mock.post(endpoint).mock(
            return_value=httpx.Response(500, json=fixtures.ERROR_RESPONSE)
        )

        with self.assertRaisesRegex(type(fixtures.ERROR), fixtures.ERROR.message):
            await self.sdk.evaluations.trigger(conversation_uuid)
        requests = cast(List[httpx.Request], [request for request, _ in endpoint_mock.calls])  # type: ignore

        [
            self.assert_requested(
                request,
                method="POST",
                endpoint=endpoint,
                body={},
            )
            for request in requests
        ]
        self.assertEqual(endpoint_mock.call_count, self.internal_options.retries)
